/*
 *  Author:  Chris Seguin
 *
 *  This software has been developed under the copyleft
 *  rules of the GNU General Public License.  Please
 *  consult the GNU General Public License for more
 *  details about use and distribution of this software.
 */
package org.acm.seguin.parser;



/*
 *  Generated By:JavaCC: Do not edit this line. ParseException.java Version 0.7pre6
 */
/**
 *  This exception is thrown when parse errors are encountered. You can
 *  explicitly create objects of this exception type by calling the method
 *  generateParseException in the generated parser. You can modify this class to
 *  customize your error reporting mechanisms so long as you retain the public
 *  fields.
 *
 *@author     Chris Seguin
 *@created    October 14, 1999
 */
public class ParseException extends Exception
{

    /**
     *  This is the last token that has been consumed successfully. If this
     *  object has been created due to a parse error, the token followng this
     *  token will (therefore) be the first error token.
     */
    public Token currentToken;

    /**
     *  Each entry in this array is an array of integers. Each array of integers
     *  represents a sequence of tokens (by their ordinal values) that is
     *  expected at this point of the parse.
     */
    public int[][] expectedTokenSequences;

    /**
     *  This is a reference to the "tokenImage" array of the generated parser
     *  within which the parse error occurred. This array is defined in the
     *  generated ...Constants interface.
     */
    public String[] tokenImage;

    /**
     *  The end of line string for this machine.
     */
    protected String eol = System.getProperty("line.separator", "\n");

    /**
     *  This variable determines which constructor was used to create this
     *  object and thereby affects the semantics of the "getMessage" method (see
     *  below).
     */
    protected boolean specialConstructor;


    /**
     *  This constructor is used by the method "generateParseException" in the
     *  generated parser. Calling this constructor generates a new object of
     *  this type with the fields "currentToken", "expectedTokenSequences", and
     *  "tokenImage" set. The boolean flag "specialConstructor" is also set to
     *  true to indicate that this constructor was used to create this object.
     *  This constructor calls its super class with the empty string to force
     *  the "toString" method of parent class "Throwable" to print the error
     *  message in the form: ParseException: <result of getMessage>
     *
     *@param  currentTokenVal            Description of Parameter
     *@param  expectedTokenSequencesVal  Description of Parameter
     *@param  tokenImageVal              Description of Parameter
     */
    public ParseException(Token currentTokenVal,
            int[][] expectedTokenSequencesVal,
            String[] tokenImageVal
            )
    {
        super("");
        specialConstructor = true;
        currentToken = currentTokenVal;
        expectedTokenSequences = expectedTokenSequencesVal;
        tokenImage = tokenImageVal;
    }


    /**
     *  The following constructors are for use by you for whatever purpose you
     *  can think of. Constructing the exception in this manner makes the
     *  exception behave in the normal way - i.e., as documented in the class
     *  "Throwable". The fields "errorToken", "expectedTokenSequences", and
     *  "tokenImage" do not contain relevant information. The JavaCC generated
     *  code does not use these constructors.
     */

    public ParseException()
    {
        super();
        specialConstructor = false;
    }


    /**
     *  Constructor for the ParseException object
     *
     *@param  message  Description of Parameter
     */
    public ParseException(String message)
    {
        super(message);
        specialConstructor = false;
    }


    /**
     *  This method has the standard behavior when this object has been created
     *  using the standard constructors. Otherwise, it uses "currentToken" and
     *  "expectedTokenSequences" to generate a parse error message and returns
     *  it. If this object has been created due to a parse error, and you do not
     *  catch it (it gets thrown from the parser), then this method is called
     *  during the printing of the final stack trace, and hence the correct
     *  error message gets displayed.
     *
     *@return    Description of the Returned Value
     */
    public String getMessage()
    {
        if (!specialConstructor) {
            return super.getMessage();
        }
        String expected = "";
        int maxSize = 0;
        for (int i = 0; i < expectedTokenSequences.length; i++) {
            if (maxSize < expectedTokenSequences[i].length) {
                maxSize = expectedTokenSequences[i].length;
            }
            for (int j = 0; j < expectedTokenSequences[i].length; j++) {
                expected += tokenImage[expectedTokenSequences[i][j]] + " ";
            }
            if (expectedTokenSequences[i][expectedTokenSequences[i].length - 1] != 0) {
                expected += "...";
            }
            expected += eol + "    ";
        }
        String retval = "Encountered \"";
        Token tok = currentToken.next;
        for (int i = 0; i < maxSize; i++) {
            if (i != 0) {
                retval += " ";
            }
            if (tok.kind == 0) {
                retval += tokenImage[0];
                break;
            }
            retval += add_escapes(tok.image);
            tok = tok.next;
        }
        retval += "\" at line " + currentToken.next.beginLine + ", column " + currentToken.next.beginColumn + "." + eol;
        if (expectedTokenSequences.length == 1) {
            retval += "Was expecting:" + eol + "    ";
        }
        else {
            retval += "Was expecting one of:" + eol + "    ";
        }
        retval += expected;
        return retval;
    }


    /**
     *  Used to convert raw characters to their escaped version when these raw
     *  version cannot be used as part of an ASCII string literal.
     *
     *@param  str  Description of Parameter
     *@return      Description of the Returned Value
     */
    protected String add_escapes(String str)
    {
        StringBuffer retval = new StringBuffer();
        char ch;
        for (int i = 0; i < str.length(); i++) {
            switch (str.charAt(i)) {
                case 0:
                    continue;
                case '\b':
                    retval.append("\\b");
                    continue;
                case '\t':
                    retval.append("\\t");
                    continue;
                case '\n':
                    retval.append("\\n");
                    continue;
                case '\f':
                    retval.append("\\f");
                    continue;
                case '\r':
                    retval.append("\\r");
                    continue;
                case '\"':
                    retval.append("\\\"");
                    continue;
                case '\'':
                    retval.append("\\\'");
                    continue;
                case '\\':
                    retval.append("\\\\");
                    continue;
                default:
                    if ((ch = str.charAt(i)) < 0x20 || ch > 0x7e) {
                        String s = "0000" + Integer.toString(ch, 16);
                        retval.append("\\u" + s.substring(s.length() - 4, s.length()));
                    }
                    else {
                        retval.append(ch);
                    }
                    continue;
            }
        }
        return retval.toString();
    }

}
